@@ -23,6 +23,9 @@
 ** See the GNU General Public License for more details.
 **
 ** $Log: acctproc.c,v $
+** Revision 1.24  2007/08/17 08:50:26  gerlof
+** Verify if private accounting used before switching off accounting.
+**
 ** Revision 1.23  2007/03/20 13:01:51  gerlof
 ** Introduction of variable supportflags.
 **
@@ -97,7 +100,7 @@
 **
 */
 
-static const char rcsid[] = "$Id: acctproc.c,v 1.23 2007/03/20 13:01:51 gerlof Exp $";
+static const char rcsid[] = "$Id: acctproc.c,v 1.24 2007/08/17 08:50:26 gerlof Exp $";
 
 #include <sys/types.h>
 #include <stdio.h>
@@ -133,6 +136,7 @@ struct pacctadm {
 	char		*name;
 	struct stat	stat;
 } pacctadm[] = {
+	{ "/var/log/account/pacct",	{0, }, },
 	{ "/var/log/pacct",		{0, }, },
 	{ "/var/account/pacct",		{0, }, }
 };
@@ -428,7 +432,8 @@ acctvers(int fd)
 void
 acctswoff(void)
 {
-	int	semid;
+	int		semid;
+	struct stat	before, after;
 
 	/*
 	** if accounting not supported, skip call
@@ -436,11 +441,6 @@ acctswoff(void)
 	if (acctfd == -1)
 		return;
 
-	/*
-	** anyhow close the accounting-file again
-	*/
-	(void) close(acctfd);	/* close account file again */
-
 	/*
 	** our private accounting-file in use?
 	*/
@@ -463,19 +463,42 @@ acctswoff(void)
 		if (semctl(semid, 1, GETVAL, 0) == SEMTOTAL)
 		{
 			/*
-			** switch off accounting and
-			** remove the directory and file
+			** switch off private accounting
+			**
+			** verify if private accounting is still in use to
+			** avoid that we switch off process accounting that
+			** has been activated manually in the meantime
 			*/
-			seteuid(0);
+			(void) fstat(acctfd, &before);
 
-			(void) acct(0);
-			(void) unlink(ACCTDIR "/" ACCTFILE);
-			(void) rmdir(ACCTDIR);
+			if ( fork() == 0 )	/* create a child and   */
+				exit(0);	/* let the child finish */
+
+			(void) wait((int *) 0); /* let the parent wait  */
+
+			(void) fstat(acctfd, &after);
+
+			if (after.st_size > before.st_size)
+			{
+				/*
+				** remove the directory and file
+				*/
+				seteuid(0);	/* get root privs again */
+
+				(void) acct(0);
+				(void) unlink(ACCTDIR "/" ACCTFILE);
+				(void) rmdir(ACCTDIR);
+			}
 		}
 
 		(void) semop(semid, &semrelse, 1);
 	}
 
+	/*
+	** anyhow close the accounting-file again
+	*/
+	(void) close(acctfd);	/* close account file again */
+
 	supportflags &= ~ACCTACTIVE;
 }
 
