@@ -115,6 +115,12 @@
 ** at runtime. 
 **
 ** $Log: atop.c,v $
+** Revision 1.31  2007/11/06 09:16:05  gerlof
+** Add keyword atopsarflags to configuration-file ~/.atoprc
+**
+** Revision 1.30  2007/08/16 11:58:35  gerlof
+** Add support for atopsar reporting.
+**
 ** Revision 1.29  2007/03/20 13:01:36  gerlof
 ** Introduction of variable supportflags.
 **
@@ -207,7 +213,7 @@
 **
 */
 
-static const char rcsid[] = "$Id: atop.c,v 1.29 2007/03/20 13:01:36 gerlof Exp $";
+static const char rcsid[] = "$Id: atop.c,v 1.31 2007/11/06 09:16:05 gerlof Exp $";
 
 #include <sys/types.h>
 #include <sys/param.h>
@@ -245,6 +251,7 @@ struct utsname	utsname;
 time_t 		pretime;	/* timing info				*/
 time_t 		curtime;	/* timing info				*/
 unsigned long	interval = 10;
+unsigned long 	sampcnt;
 char		screen;
 char		acctactive;	/* accounting active (boolean)		*/
 char		rawname[RAWNAMESZ];
@@ -267,7 +274,6 @@ struct visualize vis = {generic_samp, generic_error,
 */
 static char		awaittrigger;	/* boolean: awaiting trigger */
 static unsigned int 	nsamples = 0xffffffff;
-static long		sampcnt;
 static char		midnightflag;
 static unsigned int	begintime, endtime;
 
@@ -288,6 +294,7 @@ void do_dskcritperc(char *);
 void do_netcritperc(char *);
 void do_swoutcritsec(char *);
 void do_almostcrit(char *);
+void do_atopsarflags(char *);
 
 static struct {
 	char	*tag;
@@ -307,6 +314,7 @@ static struct {
 	{	"netcritperc",		do_netcritperc		},
 	{	"swoutcritsec",		do_swoutcritsec		},
 	{	"almostcrit",		do_almostcrit		},
+	{	"atopsarflags",		do_atopsarflags		},
 };
 
 /*
@@ -413,6 +421,19 @@ main(int argc, char *argv[])
 		}
 	}
 
+	/*
+	** check if we are supposed to behave as 'atopsar'
+	** i.e. system statistics only
+	*/
+	if ( (p = strrchr(argv[0], '/')))
+		p++;
+	else
+		p = argv[0];
+
+	if ( strcmp(p, "atopsar") == 0)
+		return atopsar(argc, argv);
+
+
 	/* 
 	** interpret command-line arguments & flags 
 	*/
@@ -810,6 +831,7 @@ prusage(char *myname)
 	printf("\tgeneric flags:\n");
 	printf("\t  -%c  show or log all processes (i.s.o. active processes "
 	                "only)\n", MALLPROC);
+	printf("\t  -P  generate parseable output for specified label(s)\n");
 
 	(*vis.show_usage)();
 
