@@ -54,6 +54,7 @@ extern struct utsname   utsname;
 extern time_t   	pretime;
 extern time_t   	curtime;
 extern unsigned long    interval;
+extern unsigned long	sampcnt;
 extern char      	screen;
 extern char		deviatonly;
 extern char		rawname[];
@@ -70,6 +71,14 @@ extern unsigned int	pagesize;
 
 extern int		supportflags;
 
+extern int		cpubadness;
+extern int		membadness;
+extern int		swpbadness;
+extern int		dskbadness;
+extern int		netbadness;
+extern int		pagbadness;
+extern int		almostcrit;
+
 /*
 ** bit-values for supportflags
 */
@@ -90,6 +99,7 @@ void		generic_usage(void);
 /*
 ** miscellaneous prototypes
 */
+int		atopsar(int, char *[]);
 char   		*convtime(time_t, char *);
 char   		*convdate(time_t, char *);
 int   		hhmm2secs(char *, unsigned int *);
