@@ -30,6 +30,22 @@
 ** --------------------------------------------------------------------------
 **
 ** $Log: deviate.c,v $
+** Revision 1.36  2007/11/05 12:13:16  gerlof
+** Match processes not only on pid, but also on start time.
+**
+** Revision 1.35  2007/11/05 11:42:47  gerlof
+** Bug-solution for new-process indicator on 64-bits machines.
+**
+** Revision 1.34  2007/08/17 09:44:59  gerlof
+** Experimental: gather info about HTTP statistics.
+**
+** Revision 1.33  2007/08/16 11:59:32  gerlof
+** Add support for atopsar reporting.
+** Concerns addition of lots of counters.
+**
+** Revision 1.32  2007/07/03 09:01:07  gerlof
+** Support Apache-statistics.
+**
 ** Revision 1.31  2007/03/20 13:02:03  gerlof
 ** Introduction of variable supportflags.
 **
@@ -125,7 +141,7 @@
 **
 */
 
-static const char rcsid[] = "$Id: deviate.c,v 1.31 2007/03/20 13:02:03 gerlof Exp $";
+static const char rcsid[] = "$Id: deviate.c,v 1.36 2007/11/05 12:13:16 gerlof Exp $";
 
 #include <sys/types.h>
 #include <sys/param.h>
@@ -182,7 +198,7 @@ deviatproc(struct pstat *aproc, int npresent,
 		/*
 		** get previous figures from process-database
 		*/
-		if ( pdb_getproc(curstat->gen.pid, &pinfo) )
+		if ( pdb_getproc(curstat->gen.pid, curstat->gen.btime, &pinfo))
 		{
 			/*
 			** process already present during the previous sample;
@@ -233,7 +249,7 @@ deviatproc(struct pstat *aproc, int npresent,
 		devstat->gen        = curstat->gen;
 
 		if (newproc)
-			devstat->gen.excode |= ~(LONG_MAX);
+			devstat->gen.excode |= ~(INT_MAX);
 
 		devstat->cpu.nice     = curstat->cpu.nice;
 		devstat->cpu.prio     = curstat->cpu.prio;
@@ -308,7 +324,8 @@ deviatproc(struct pstat *aproc, int npresent,
 
 		if (curstat->gen.pid)	/* acctrecord contains pid? */
 		{
-			if ( pdb_getproc(curstat->gen.pid, &pinfo) )
+			if ( pdb_getproc(curstat->gen.pid,
+			                 curstat->gen.btime, &pinfo))
 					prestat = pinfo->pstat;
 				else
 					memset(&prestat, 0, sizeof(prestat));
@@ -347,7 +364,7 @@ deviatproc(struct pstat *aproc, int npresent,
 			devstat->gen.pid    = prestat.gen.pid;
 
 		if (!prestat.gen.pid)
-			devstat->gen.excode |= ~(LONG_MAX);
+			devstat->gen.excode |= ~(INT_MAX);
 
 		strcpy(devstat->gen.cmdline, prestat.gen.cmdline);
 
@@ -489,10 +506,12 @@ void
 deviatsyst(struct sstat *cur, struct sstat *pre, struct sstat *dev)
 {
 	register int	i;
+	count_t		*cdev, *ccur, *cpre;
 
+	dev->cpu.nrcpu     = cur->cpu.nrcpu;
 	dev->cpu.devint    = subcount(cur->cpu.devint, pre->cpu.devint);
 	dev->cpu.csw       = subcount(cur->cpu.csw,    pre->cpu.csw);
-	dev->cpu.nrcpu     = cur->cpu.nrcpu;
+	dev->cpu.nprocs    = subcount(cur->cpu.nprocs, pre->cpu.nprocs);
 
 	dev->cpu.all.stime = subcount(cur->cpu.all.stime, pre->cpu.all.stime);
 	dev->cpu.all.utime = subcount(cur->cpu.all.utime, pre->cpu.all.utime);
@@ -553,14 +572,87 @@ deviatsyst(struct sstat *cur, struct sstat *pre, struct sstat *dev)
 	dev->mem.allocstall	= subcount(cur->mem.allocstall,
 				                         pre->mem.allocstall);
 
-	dev->net.ipin 		= subcount(cur->net.ipin,    pre->net.ipin);
-	dev->net.ipout 		= subcount(cur->net.ipout,   pre->net.ipout);
-	dev->net.ipindel 	= subcount(cur->net.ipindel, pre->net.ipindel);
-	dev->net.ipfrw 		= subcount(cur->net.ipfrw,   pre->net.ipfrw);
-	dev->net.tcpin 		= subcount(cur->net.tcpin,   pre->net.tcpin);
-	dev->net.tcpout		= subcount(cur->net.tcpout,  pre->net.tcpout);
-	dev->net.udpin 		= subcount(cur->net.udpin,   pre->net.udpin);
-	dev->net.udpout		= subcount(cur->net.udpout,  pre->net.udpout);
+	/*
+	** structures with network-related counters are considered
+	** as tabels of frequency-counters that have to be subtracte;
+	** values that do not represent a frequency are corrected afterwards
+	*/
+	for (cdev = (count_t *)&dev->net.ipv4,
+	     ccur = (count_t *)&cur->net.ipv4,
+	     cpre = (count_t *)&pre->net.ipv4,
+	     i    = 0;
+		i < (sizeof dev->net.ipv4 / sizeof(count_t));
+	     		cdev++, ccur++, cpre++, i++)
+				*cdev = *ccur - *cpre;
+	
+	dev->net.ipv4.Forwarding = cur->net.ipv4.Forwarding;
+	dev->net.ipv4.DefaultTTL = cur->net.ipv4.DefaultTTL;
+
+		/* ------------- */
+
+	for (cdev = (count_t *)&dev->net.icmpv4,
+	     ccur = (count_t *)&cur->net.icmpv4,
+	     cpre = (count_t *)&pre->net.icmpv4,
+	     i    = 0;
+		i < (sizeof dev->net.icmpv4 / sizeof(count_t));
+	     		cdev++, ccur++, cpre++, i++)
+				*cdev = *ccur - *cpre;
+
+		/* ------------- */
+
+	for (cdev = (count_t *)&dev->net.udpv4,
+	     ccur = (count_t *)&cur->net.udpv4,
+	     cpre = (count_t *)&pre->net.udpv4,
+	     i    = 0;
+		i < (sizeof dev->net.udpv4 / sizeof(count_t));
+	     		cdev++, ccur++, cpre++, i++)
+				*cdev = *ccur - *cpre;
+
+		/* ------------- */
+
+	for (cdev = (count_t *)&dev->net.ipv6,
+	     ccur = (count_t *)&cur->net.ipv6,
+	     cpre = (count_t *)&pre->net.ipv6,
+	     i    = 0;
+		i < (sizeof dev->net.ipv6 / sizeof(count_t));
+	     		cdev++, ccur++, cpre++, i++)
+				*cdev = *ccur - *cpre;
+
+		/* ------------- */
+
+	for (cdev = (count_t *)&dev->net.icmpv6,
+	     ccur = (count_t *)&cur->net.icmpv6,
+	     cpre = (count_t *)&pre->net.icmpv6,
+	     i    = 0;
+		i < (sizeof dev->net.icmpv6 / sizeof(count_t));
+	     		cdev++, ccur++, cpre++, i++)
+				*cdev = *ccur - *cpre;
+
+		/* ------------- */
+
+	for (cdev = (count_t *)&dev->net.udpv6,
+	     ccur = (count_t *)&cur->net.udpv6,
+	     cpre = (count_t *)&pre->net.udpv6,
+	     i    = 0;
+		i < (sizeof dev->net.udpv6 / sizeof(count_t));
+	     		cdev++, ccur++, cpre++, i++)
+				*cdev = *ccur - *cpre;
+
+		/* ------------- */
+
+	for (cdev = (count_t *)&dev->net.tcp,
+	     ccur = (count_t *)&cur->net.tcp,
+	     cpre = (count_t *)&pre->net.tcp,
+	     i    = 0;
+		i < (sizeof dev->net.tcp / sizeof(count_t));
+	     		cdev++, ccur++, cpre++, i++)
+				*cdev = *ccur - *cpre;
+
+	dev->net.tcp.RtoAlgorithm = cur->net.tcp.RtoAlgorithm;
+	dev->net.tcp.RtoMin       = cur->net.tcp.RtoMin;
+	dev->net.tcp.RtoMax       = cur->net.tcp.RtoMax;
+	dev->net.tcp.MaxConn      = cur->net.tcp.MaxConn;
+	dev->net.tcp.CurrEstab    = cur->net.tcp.CurrEstab;
 
 	/*
 	** calculate deviations for interfaces
@@ -622,13 +714,38 @@ deviatsyst(struct sstat *cur, struct sstat *pre, struct sstat *dev)
 		strcpy(dev->intf.intf[i].name, cur->intf.intf[i].name);
 
 		dev->intf.intf[i].rbyte = subcount(cur->intf.intf[i].rbyte,
-           	                                  pre->intf.intf[i].rbyte);
+           	                                   pre->intf.intf[i].rbyte);
 		dev->intf.intf[i].rpack = subcount(cur->intf.intf[i].rpack,
-		                                  pre->intf.intf[i].rpack);
+		                                   pre->intf.intf[i].rpack);
+		dev->intf.intf[i].rerrs = subcount(cur->intf.intf[i].rerrs,
+		                                   pre->intf.intf[i].rerrs);
+		dev->intf.intf[i].rdrop = subcount(cur->intf.intf[i].rdrop,
+		                                   pre->intf.intf[i].rdrop);
+		dev->intf.intf[i].rfifo = subcount(cur->intf.intf[i].rfifo,
+		                                   pre->intf.intf[i].rfifo);
+		dev->intf.intf[i].rframe= subcount(cur->intf.intf[i].rframe,
+		                                   pre->intf.intf[i].rframe);
+		dev->intf.intf[i].rcompr= subcount(cur->intf.intf[i].rcompr,
+		                                   pre->intf.intf[i].rcompr);
+		dev->intf.intf[i].rmultic=subcount(cur->intf.intf[i].rmultic,
+		                                   pre->intf.intf[i].rmultic);
+
 		dev->intf.intf[i].sbyte = subcount(cur->intf.intf[i].sbyte,
-		                                  pre->intf.intf[i].sbyte);
+		                                   pre->intf.intf[i].sbyte);
 		dev->intf.intf[i].spack = subcount(cur->intf.intf[i].spack,
-		                                  pre->intf.intf[i].spack);
+		                                   pre->intf.intf[i].spack);
+		dev->intf.intf[i].serrs = subcount(cur->intf.intf[i].serrs,
+		                                   pre->intf.intf[i].serrs);
+		dev->intf.intf[i].sdrop = subcount(cur->intf.intf[i].sdrop,
+		                                   pre->intf.intf[i].sdrop);
+		dev->intf.intf[i].sfifo = subcount(cur->intf.intf[i].sfifo,
+		                                   pre->intf.intf[i].sfifo);
+		dev->intf.intf[i].scollis= subcount(cur->intf.intf[i].scollis,
+		                                   pre->intf.intf[i].scollis);
+		dev->intf.intf[i].scarrier= subcount(cur->intf.intf[i].scarrier,
+		                                   pre->intf.intf[i].scarrier);
+		dev->intf.intf[i].scompr= subcount(cur->intf.intf[i].scompr,
+		                                   pre->intf.intf[i].scompr);
 
 		dev->intf.intf[i].speed 	= pre->intf.intf[i].speed;
 		dev->intf.intf[i].duplex	= pre->intf.intf[i].duplex;
@@ -651,21 +768,42 @@ deviatsyst(struct sstat *cur, struct sstat *pre, struct sstat *dev)
 		strcpy(dev->xdsk.xdsk[i].name, cur->xdsk.xdsk[i].name);
 
 		dev->xdsk.xdsk[i].nread  = subcount(cur->xdsk.xdsk[i].nread,
-		                                   pre->xdsk.xdsk[i].nread);
+		                                    pre->xdsk.xdsk[i].nread);
 		dev->xdsk.xdsk[i].nwrite = subcount(cur->xdsk.xdsk[i].nwrite,
-		                                   pre->xdsk.xdsk[i].nwrite);
-		dev->xdsk.xdsk[i].nrblk  = subcount(cur->xdsk.xdsk[i].nrblk,
-		                                   pre->xdsk.xdsk[i].nrblk);
-		dev->xdsk.xdsk[i].nwblk  = subcount(cur->xdsk.xdsk[i].nwblk,
-		                                   pre->xdsk.xdsk[i].nwblk);
+		                                    pre->xdsk.xdsk[i].nwrite);
+		dev->xdsk.xdsk[i].nrsect = subcount(cur->xdsk.xdsk[i].nrsect,
+		                                    pre->xdsk.xdsk[i].nrsect);
+		dev->xdsk.xdsk[i].nwsect = subcount(cur->xdsk.xdsk[i].nwsect,
+		                                    pre->xdsk.xdsk[i].nwsect);
 		dev->xdsk.xdsk[i].io_ms  = subcount(cur->xdsk.xdsk[i].io_ms,
-		                                   pre->xdsk.xdsk[i].io_ms);
+		                                    pre->xdsk.xdsk[i].io_ms);
 		dev->xdsk.xdsk[i].avque  = subcount(cur->xdsk.xdsk[i].avque,
-		                                   pre->xdsk.xdsk[i].avque);
+		                                    pre->xdsk.xdsk[i].avque);
 	}
 
 	dev->xdsk.xdsk[i].name[0] = '\0';
 	dev->xdsk.nrxdsk = i;
+
+	/*
+	** application-specific counters
+	*/
+#if	HTTPSTATS
+	if (cur->www.uptime >= pre->www.uptime)
+	{
+		dev->www.accesses  = subcount(cur->www.accesses,
+                                              pre->www.accesses);
+		dev->www.totkbytes = subcount(cur->www.totkbytes,
+                                              pre->www.totkbytes);
+	}
+	else
+	{
+		dev->www.accesses  = cur->www.accesses;
+		dev->www.totkbytes = cur->www.totkbytes;
+	}
+
+	dev->www.bworkers  = cur->www.bworkers;
+	dev->www.iworkers  = cur->www.iworkers;
+#endif
 }
 
 /*
