@@ -31,6 +31,10 @@
 ** --------------------------------------------------------------------------
 **
 ** $Log: photoproc.c,v $
+** Revision 1.29  2007/11/05 12:26:10  gerlof
+** Detect disappearing /proc/stat file  when process exits
+** (credits: Rene Rebe).
+**
 ** Revision 1.28  2007/03/27 10:53:59  gerlof
 ** Bug-solution: only allow IOSTAT when patches are not installed.
 **
@@ -120,7 +124,7 @@
 **
 */
 
-static const char rcsid[] = "$Id: photoproc.c,v 1.28 2007/03/27 10:53:59 gerlof Exp $";
+static const char rcsid[] = "$Id: photoproc.c,v 1.29 2007/11/05 12:26:10 gerlof Exp $";
 
 #include <sys/types.h>
 #include <sys/param.h>
@@ -304,7 +308,11 @@ fillproc(struct pstat *curproc)
 	if ( (fp = fopen("stat", "r")) == NULL)
 		return 0;
 
-	(void) fgets(line, sizeof line, fp);
+	if (fgets(line, sizeof line, fp) == NULL)
+	{
+		fclose(fp);
+		return 0;
+	}
 
 	sscanf(line, "%d", &pid);		/* fetch pid */
 
