@@ -110,7 +110,7 @@ struct pinfo {
 /*
 ** prototypes of process-database functions
 */
-int		pdb_getproc(int, struct pinfo **);
+int		pdb_getproc(int, time_t, struct pinfo **);
 void		pdb_addproc(int, struct pinfo *);
 int		pdb_delproc(int);
 int		pdb_newproc(struct pinfo **);
