@@ -30,6 +30,16 @@
 ** --------------------------------------------------------------------------
 **
 ** $Log: photosyst.c,v $
+** Revision 1.29  2007/08/17 09:45:44  gerlof
+** Experimental: gather info about HTTP statistics.
+**
+** Revision 1.28  2007/08/16 12:00:49  gerlof
+** Add support for atopsar reporting.
+** Gather more counters, mainly related to networking.
+**
+** Revision 1.27  2007/07/03 09:01:56  gerlof
+** Support Apache-statistics.
+**
 ** Revision 1.26  2007/02/13 10:32:28  gerlof
 ** Removal of external declarations.
 **
@@ -111,7 +121,7 @@
 **
 */
 
-static const char rcsid[] = "$Id: photosyst.c,v 1.26 2007/02/13 10:32:28 gerlof Exp $";
+static const char rcsid[] = "$Id: photosyst.c,v 1.29 2007/08/17 09:45:44 gerlof Exp $";
 
 #include <sys/types.h>
 #include <stdio.h>
@@ -123,6 +133,11 @@ static const char rcsid[] = "$Id: photosyst.c,v 1.26 2007/02/13 10:32:28 gerlof
 #include <sys/wait.h>
 #include <time.h>
 #include <signal.h>
+#include <string.h>
+
+#include <sys/socket.h>
+#include <netinet/in.h>
+#include <netdb.h>
 
 #include "atop.h"
 #include "photosyst.h"
@@ -131,15 +146,95 @@ static const char rcsid[] = "$Id: photosyst.c,v 1.26 2007/02/13 10:32:28 gerlof
 
 static int	isrealdisk(char *, char *, int);
 
+static struct ipv6_stats	ipv6_tmp;
+static struct icmpv6_stats	icmpv6_tmp;
+static struct udpv6_stats	udpv6_tmp;
+
+struct v6tab {
+	char 	*nam;
+	count_t *val;
+};
+
+static struct v6tab 		v6tab[] = {
+    {"Ip6InReceives",		&ipv6_tmp.Ip6InReceives,                     },
+    {"Ip6InHdrErrors",		&ipv6_tmp.Ip6InHdrErrors,                    },
+    {"Ip6InTooBigErrors",	&ipv6_tmp.Ip6InTooBigErrors,                 },
+    {"Ip6InNoRoutes",		&ipv6_tmp.Ip6InNoRoutes,                     },
+    {"Ip6InAddrErrors",		&ipv6_tmp.Ip6InAddrErrors,                   },
+    {"Ip6InUnknownProtos",	&ipv6_tmp.Ip6InUnknownProtos,                },
+    {"Ip6InTruncatedPkts",	&ipv6_tmp.Ip6InTruncatedPkts,                },
+    {"Ip6InDiscards",		&ipv6_tmp.Ip6InDiscards,                     },
+    {"Ip6InDelivers",		&ipv6_tmp.Ip6InDelivers,                     },
+    {"Ip6OutForwDatagrams",	&ipv6_tmp.Ip6OutForwDatagrams,               },
+    {"Ip6OutRequests",		&ipv6_tmp.Ip6OutRequests,                    },
+    {"Ip6OutDiscards",		&ipv6_tmp.Ip6OutDiscards,                    },
+    {"Ip6OutNoRoutes",		&ipv6_tmp.Ip6OutNoRoutes,                    },
+    {"Ip6ReasmTimeout",		&ipv6_tmp.Ip6ReasmTimeout,                   },
+    {"Ip6ReasmReqds",		&ipv6_tmp.Ip6ReasmReqds,                     },
+    {"Ip6ReasmOKs",		&ipv6_tmp.Ip6ReasmOKs,                       },
+    {"Ip6ReasmFails",		&ipv6_tmp.Ip6ReasmFails,                     },
+    {"Ip6FragOKs",		&ipv6_tmp.Ip6FragOKs,                        },
+    {"Ip6FragFails",		&ipv6_tmp.Ip6FragFails,                      },
+    {"Ip6FragCreates",		&ipv6_tmp.Ip6FragCreates,                    },
+    {"Ip6InMcastPkts",		&ipv6_tmp.Ip6InMcastPkts,                    },
+    {"Ip6OutMcastPkts",		&ipv6_tmp.Ip6OutMcastPkts,                   },
+ 
+    {"Icmp6InMsgs",		&icmpv6_tmp.Icmp6InMsgs,                     },
+    {"Icmp6InErrors",		&icmpv6_tmp.Icmp6InErrors,                   },
+    {"Icmp6InDestUnreachs",	&icmpv6_tmp.Icmp6InDestUnreachs,             },
+    {"Icmp6InPktTooBigs",	&icmpv6_tmp.Icmp6InPktTooBigs,               },
+    {"Icmp6InTimeExcds",	&icmpv6_tmp.Icmp6InTimeExcds,                },
+    {"Icmp6InParmProblems",	&icmpv6_tmp.Icmp6InParmProblems,             },
+    {"Icmp6InEchos",		&icmpv6_tmp.Icmp6InEchos,                    },
+    {"Icmp6InEchoReplies",	&icmpv6_tmp.Icmp6InEchoReplies,              },
+    {"Icmp6InGroupMembQueries",	&icmpv6_tmp.Icmp6InGroupMembQueries,         },
+    {"Icmp6InGroupMembResponses",
+				&icmpv6_tmp.Icmp6InGroupMembResponses,       },
+    {"Icmp6InGroupMembReductions",
+				&icmpv6_tmp.Icmp6InGroupMembReductions,      },
+    {"Icmp6InRouterSolicits",	&icmpv6_tmp.Icmp6InRouterSolicits,           },
+    {"Icmp6InRouterAdvertisements",
+				&icmpv6_tmp.Icmp6InRouterAdvertisements,     },
+    {"Icmp6InNeighborSolicits",	&icmpv6_tmp.Icmp6InNeighborSolicits,         },
+    {"Icmp6InNeighborAdvertisements",
+				&icmpv6_tmp.Icmp6InNeighborAdvertisements,   },
+    {"Icmp6InRedirects",	&icmpv6_tmp.Icmp6InRedirects,                },
+    {"Icmp6OutMsgs",		&icmpv6_tmp.Icmp6OutMsgs,                    },
+    {"Icmp6OutDestUnreachs",	&icmpv6_tmp.Icmp6OutDestUnreachs,            },
+    {"Icmp6OutPktTooBigs",	&icmpv6_tmp.Icmp6OutPktTooBigs,              },
+    {"Icmp6OutTimeExcds",	&icmpv6_tmp.Icmp6OutTimeExcds,               },
+    {"Icmp6OutParmProblems",	&icmpv6_tmp.Icmp6OutParmProblems,            },
+    {"Icmp6OutEchoReplies",	&icmpv6_tmp.Icmp6OutEchoReplies,             },
+    {"Icmp6OutRouterSolicits",	&icmpv6_tmp.Icmp6OutRouterSolicits,          },
+    {"Icmp6OutNeighborSolicits",&icmpv6_tmp.Icmp6OutNeighborSolicits,        },
+    {"Icmp6OutNeighborAdvertisements",
+				&icmpv6_tmp.Icmp6OutNeighborAdvertisements,  },
+    {"Icmp6OutRedirects",	&icmpv6_tmp.Icmp6OutRedirects,               },
+    {"Icmp6OutGroupMembResponses",
+				&icmpv6_tmp.Icmp6OutGroupMembResponses,      },
+    {"Icmp6OutGroupMembReductions",
+				&icmpv6_tmp.Icmp6OutGroupMembReductions,     },
+
+    {"Udp6InDatagrams",		&udpv6_tmp.Udp6InDatagrams,                   },
+    {"Udp6NoPorts",		&udpv6_tmp.Udp6NoPorts,                       },
+    {"Udp6InErrors",		&udpv6_tmp.Udp6InErrors,                      },
+    {"Udp6OutDatagrams",	&udpv6_tmp.Udp6OutDatagrams,                  },
+};
+
+static int	v6tab_entries = sizeof(v6tab)/sizeof(struct v6tab);
+
 void
 photosyst(struct sstat *si)
 {
-	register int	i, nr, nlines;
+	register int	i, nr;
 	count_t		cnts[MAXCNT];
 	float		lavg1, lavg5, lavg15;
 	FILE 		*fp;
 	char		linebuf[1024], nam[64], origdir[1024];
 	static char	part_stats = 1; /* per-partition statistics ? */
+#if	HTTPSTATS
+	static int	wwwvalid = 1;
+#endif
 
 	memset(si, 0, sizeof(struct sstat));
 
@@ -155,30 +250,13 @@ photosyst(struct sstat *si)
 		while ( fgets(linebuf, sizeof(linebuf), fp) != NULL)
 		{
 			nr = sscanf(linebuf,
-			 "%s   %lld %lld %lld %lld %lld %lld %lld %lld %lld "
-			 "%lld %lld %lld %lld %lld %lld %lld %lld %lld %lld "
-			 "%lld %lld %lld %lld %lld %lld %lld %lld %lld %lld "
-			 "%lld %lld %lld %lld %lld %lld %lld %lld %lld %lld "
-			 "%lld %lld %lld %lld %lld %lld %lld %lld %lld %lld "
-			 "%lld %lld %lld %lld %lld %lld %lld %lld %lld %lld "
-			 "%lld %lld %lld %lld %lld ",
+			            "%s   %lld %lld %lld %lld %lld %lld %lld "
+			            "%lld %lld %lld %lld %lld %lld %lld %lld ",
 			  	nam,
 			  	&cnts[0],  &cnts[1],  &cnts[2],  &cnts[3],
 			  	&cnts[4],  &cnts[5],  &cnts[6],  &cnts[7],
 			  	&cnts[8],  &cnts[9],  &cnts[10], &cnts[11],
-			  	&cnts[12], &cnts[13], &cnts[14], &cnts[15],
-			  	&cnts[16], &cnts[17], &cnts[18], &cnts[19],
-			  	&cnts[20], &cnts[21], &cnts[22], &cnts[23],
-			  	&cnts[24], &cnts[25], &cnts[26], &cnts[27],
-			  	&cnts[28], &cnts[29], &cnts[30], &cnts[31],
-			  	&cnts[32], &cnts[33], &cnts[34], &cnts[35],
-			  	&cnts[36], &cnts[37], &cnts[38], &cnts[39],
-			  	&cnts[40], &cnts[41], &cnts[42], &cnts[43],
-			  	&cnts[44], &cnts[45], &cnts[46], &cnts[47],
-			  	&cnts[48], &cnts[49], &cnts[50], &cnts[51],
-			  	&cnts[52], &cnts[53], &cnts[54], &cnts[55],
-			  	&cnts[56], &cnts[57], &cnts[58], &cnts[59],
-			  	&cnts[60], &cnts[61], &cnts[62], &cnts[63]);
+			  	&cnts[12], &cnts[13], &cnts[14]);
 
 			if (nr < 2)		/* headerline ? --> skip */
 				continue;
@@ -238,6 +316,12 @@ photosyst(struct sstat *si)
 				continue;
 			}
 
+			if ( strcmp("processes", nam) == EQ)
+			{
+				si->cpu.nprocs	= cnts[0];
+				continue;
+			}
+
 			if ( strcmp("swap", nam) == EQ)   /* < 2.6 */
 			{
 				si->mem.swins	= cnts[0];
@@ -449,16 +533,66 @@ photosyst(struct sstat *si)
 
 	/*
 	** gather network-related statistics
+ 	** 	- interface stats from the file /proc/net/dev
  	** 	- IPv4      stats from the file /proc/net/snmp
  	** 	- IPv6      stats from the file /proc/net/snmp6
- 	** 	- interface stats from the file /proc/net/dev
+	*/
+
+	/*
+	** interface statistics
+	*/
+	if ( (fp = fopen("net/dev", "r")) != NULL)
+	{
+		char *cp;
+
+		i = 0;
+
+		while ( fgets(linebuf, sizeof(linebuf), fp) != NULL)
+		{
+			if ( (cp = strchr(linebuf, ':')) != NULL)
+				*cp = ' ';      /* substitute ':' by space */
+
+			nr = sscanf(linebuf,
+                                    "%15s %lld %lld %lld %lld %lld %lld %lld "
+                                    "%lld %lld %lld %lld %lld %lld %lld %lld "
+                                    "%lld\n",
+				  si->intf.intf[i].name,
+				&(si->intf.intf[i].rbyte),
+				&(si->intf.intf[i].rpack),
+				&(si->intf.intf[i].rerrs),
+				&(si->intf.intf[i].rdrop),
+				&(si->intf.intf[i].rfifo),
+				&(si->intf.intf[i].rframe),
+				&(si->intf.intf[i].rcompr),
+				&(si->intf.intf[i].rmultic),
+				&(si->intf.intf[i].sbyte),
+				&(si->intf.intf[i].spack),
+				&(si->intf.intf[i].serrs),
+				&(si->intf.intf[i].sdrop),
+				&(si->intf.intf[i].sfifo),
+				&(si->intf.intf[i].scollis),
+				&(si->intf.intf[i].scarrier),
+				&(si->intf.intf[i].scompr));
+
+			if (nr == 17)	/* skip header & lines without stats */
+			{
+				if (++i >= MAXINTF-1)
+					break;
+			}
+		}
+
+		si->intf.intf[i].name[0] = '\0'; /* set terminator for table */
+		si->intf.nrintf = i;
+
+		fclose(fp);
+	}
+
+	/*
+	** IP version 4 statistics
 	*/
 	if ( (fp = fopen("net/snmp", "r")) != NULL)
 	{
-		nlines = 3;
-
-		while ( fgets(linebuf, sizeof(linebuf), fp) != NULL &&
-		                                               nlines > 0)
+		while ( fgets(linebuf, sizeof(linebuf), fp) != NULL)
 		{
 			nr = sscanf(linebuf,
 			 "%s   %lld %lld %lld %lld %lld %lld %lld %lld %lld "
@@ -481,39 +615,49 @@ photosyst(struct sstat *si)
 			if (nr < 2)		/* headerline ? --> skip */
 				continue;
 
-			if ( strcmp("Ip:", nam) == EQ)
+			if ( strcmp("Ip:", nam) == 0)
 			{
-				si->net.ipin	= cnts[2];
-				si->net.ipout	= cnts[9];
-				si->net.ipindel	= cnts[8];
-				si->net.ipfrw	= cnts[5];
-				nlines--;
+				memcpy(&si->net.ipv4, cnts,
+						sizeof si->net.ipv4);
 				continue;
 			}
-
-			if ( strcmp("Tcp:", nam) == EQ)
+	
+			if ( strcmp("Icmp:", nam) == 0)
 			{
-				si->net.tcpin	= cnts[9];
-				si->net.tcpout	= cnts[10];
-				nlines--;
+				memcpy(&si->net.icmpv4, cnts,
+						sizeof si->net.icmpv4);
 				continue;
 			}
-
-			if ( strcmp("Udp:", nam) == EQ)
+	
+			if ( strcmp("Tcp:", nam) == 0)
 			{
-				si->net.udpin	= cnts[0];
-				si->net.udpout	= cnts[3];
-				nlines--;
+				memcpy(&si->net.tcp, cnts,
+						sizeof si->net.tcp);
+				continue;
+			}
+	
+			if ( strcmp("Udp:", nam) == 0)
+			{
+				memcpy(&si->net.udpv4, cnts,
+						sizeof si->net.udpv4);
 				continue;
 			}
 		}
-
+	
 		fclose(fp);
 	}
 
+	/*
+	** IP version 6 statistics
+	*/
+	memset(&ipv6_tmp,   0, sizeof ipv6_tmp);
+	memset(&icmpv6_tmp, 0, sizeof icmpv6_tmp);
+	memset(&udpv6_tmp,  0, sizeof udpv6_tmp);
+
 	if ( (fp = fopen("net/snmp6", "r")) != NULL)
 	{
 		count_t	countval;
+		int	cur = 0;
 
 		/*
 		** one name-value pair per line
@@ -525,66 +669,27 @@ photosyst(struct sstat *si)
 			if (nr < 2)		/* unexpected line ? --> skip */
 				continue;
 
-			if ( memcmp("Icmp6", nam, 5) == EQ)
-				continue;
+		   	if (strcmp(v6tab[cur].nam, nam) == 0)
+		   	{
+		   		*(v6tab[cur].val) = countval;
+		   	}
+		   	else
+		   	{
+		   		for (cur=0; cur < v6tab_entries; cur++)
+					if (strcmp(v6tab[cur].nam, nam) == 0)
+						break;
 
-			if ( memcmp("Ip6", nam, 3) == EQ)
-			{
-				if      ( strcmp("Ip6InReceives", nam) == EQ)
-					si->net.ipin	+= countval;
-				else if ( strcmp("Ip6OutRequests", nam) == EQ)
-					si->net.ipout	+= countval;
-				else if ( strcmp("Ip6InDelivers", nam) == EQ)
-					si->net.ipindel	+= countval;
-				else if (strcmp("Ip6OutForwDatagrams",nam)==EQ)
-					si->net.ipfrw	+= countval;
-
-				continue;
+				if (cur < v6tab_entries) /* found ? */
+		   			*(v6tab[cur].val) = countval;
 			}
 
-			if ( memcmp("Udp6", nam, 4) == EQ)
-			{
-				if      ( strcmp("Udp6InDatagrams", nam) == EQ)
-					si->net.udpin  += countval;
-				else if ( strcmp("Udp6OutDatagrams", nam) ==EQ)
-					si->net.udpout += countval;
-
-				continue;
-			}
+			if (++cur >= v6tab_entries)
+				cur = 0;
 		}
 
-		fclose(fp);
-	}
-
-	if ( (fp = fopen("net/dev", "r")) != NULL)
-	{
-		char *cp;
-
-		i = 0;
-
-		while ( fgets(linebuf, sizeof(linebuf), fp) != NULL)
-		{
-			if ( (cp = strchr(linebuf, ':')) != NULL)
-				*cp = ' ';      /* substitute ':' by space */
-
-			nr = sscanf(linebuf,
-		 	    "%15s  %lld %lld %*d %*d %*d %*d %*d "
-			    "%*d %lld %lld %*d %*d %*d %*d %*d %*d",
-				  si->intf.intf[i].name,
-				&(si->intf.intf[i].rbyte),
-				&(si->intf.intf[i].rpack),
-				&(si->intf.intf[i].sbyte),
-				&(si->intf.intf[i].spack));
-
-			if (nr == 5)	/* skip header & lines without stats */
-			{
-				if (++i >= MAXINTF-1)
-					break;
-			}
-		}
-
-		si->intf.intf[i].name[0] = '\0'; /* set terminator for table */
-		si->intf.nrintf = i;
+		memcpy(&si->net.ipv6,   &ipv6_tmp,   sizeof ipv6_tmp);
+		memcpy(&si->net.icmpv6, &icmpv6_tmp, sizeof icmpv6_tmp);
+		memcpy(&si->net.udpv6,  &udpv6_tmp,  sizeof udpv6_tmp);
 
 		fclose(fp);
 	}
@@ -605,9 +710,9 @@ photosyst(struct sstat *si)
 			      "%lld %*d %lld %*d %*d %lld %lld",
 			        diskname,
 				&(si->xdsk.xdsk[i].nread),
-				&(si->xdsk.xdsk[i].nrblk),
+				&(si->xdsk.xdsk[i].nrsect),
 				&(si->xdsk.xdsk[i].nwrite),
-				&(si->xdsk.xdsk[i].nwblk),
+				&(si->xdsk.xdsk[i].nwsect),
 				&(si->xdsk.xdsk[i].io_ms),
 				&(si->xdsk.xdsk[i].avque) );
 
@@ -653,9 +758,9 @@ photosyst(struct sstat *si)
 			      "%lld %*d %lld %*d %*d %lld %lld",
 				diskname,
 				&(si->xdsk.xdsk[i].nread),
-				&(si->xdsk.xdsk[i].nrblk),
+				&(si->xdsk.xdsk[i].nrsect),
 				&(si->xdsk.xdsk[i].nwrite),
-				&(si->xdsk.xdsk[i].nwblk),
+				&(si->xdsk.xdsk[i].nwsect),
 				&(si->xdsk.xdsk[i].io_ms),
 				&(si->xdsk.xdsk[i].avque) );
 
@@ -683,6 +788,14 @@ photosyst(struct sstat *si)
 	}
 
 	chdir(origdir);
+
+	/*
+	** fetch application-specific counters
+	*/
+#if	HTTPSTATS
+	if ( wwwvalid)
+		wwwvalid = getwwwstat(80, &(si->www));
+#endif
 }
 
 /*
@@ -822,3 +935,120 @@ getbootlinux(long hertz)
 
 	return boottime;
 }
+
+#if	HTTPSTATS
+/*
+** retrieve statistics from local HTTP daemons
+** via http://localhost/server-status?auto
+*/
+int
+getwwwstat(unsigned short port, struct wwwstat *wp)
+{
+	int 			sockfd, tobefound;
+	FILE			*sockfp;
+	struct sockaddr_in	sockname;
+	char			linebuf[4096];
+	char			label[512];
+	long long		value;
+
+	memset(wp, 0, sizeof *wp);
+
+	/*
+	** allocate a socket and connect to the local HTTP daemon
+	*/
+	if ( (sockfd = socket(AF_INET, SOCK_STREAM, 0)) == -1)
+		return 0;
+
+	sockname.sin_family		= AF_INET;
+	sockname.sin_addr.s_addr	= htonl(INADDR_LOOPBACK);
+	sockname.sin_addr.s_addr	= htonl(0xc36ce51e);
+	sockname.sin_port		= htons(port);
+
+	if ( connect(sockfd, (struct sockaddr *) &sockname,
+						sizeof sockname) == -1)
+	{
+		close(sockfd);
+		return 0;
+	}
+
+	/*
+	** write a GET-request for /server-status
+	*/
+	if ( write(sockfd, HTTPREQ, sizeof HTTPREQ) < sizeof HTTPREQ)
+	{
+		close(sockfd);
+		return 0;
+	}
+
+	/*
+	** remap socket descriptor to a stream to allow stdio calls
+	*/
+	sockfp = fdopen(sockfd, "r+");
+
+	/*
+	** read response line by line
+	*/
+	tobefound = 5;		/* number of values to be searched */
+
+	while ( fgets(linebuf, sizeof linebuf, sockfp) && tobefound)
+	{
+		/*
+		** handle line containing status code
+		*/
+		if ( strncmp(linebuf, "HTTP/", 5) == 0)
+		{
+			sscanf(linebuf, "%511s %lld %*s\n", label, &value);
+
+			if (value != 200)	/* HTTP-request okay? */
+			{
+				fclose(sockfp);
+				close(sockfd);
+				return 0;
+			}
+
+			continue;
+		}
+
+		/*
+		** decode line and search for the required counters
+		*/
+		if (sscanf(linebuf, "%511[^:]: %lld\n", label, &value) == 2)
+		{
+			if ( strcmp(label, "Total Accesses") == 0)
+			{
+				wp->accesses = value;
+				tobefound--;
+			}
+
+			if ( strcmp(label, "Total kBytes") == 0)
+			{
+				wp->totkbytes = value;
+				tobefound--;
+			}
+
+			if ( strcmp(label, "Uptime") == 0)
+			{
+				wp->uptime = value;
+				tobefound--;
+			}
+
+			if ( strcmp(label, "BusyWorkers") == 0)
+			{
+				wp->bworkers = value;
+				tobefound--;
+			}
+
+			if ( strcmp(label, "IdleWorkers") == 0)
+			{
+				wp->iworkers = value;
+				tobefound--;
+			}
+		}
+	}
+
+	fclose(sockfp);
+	close(sockfd);
+
+	return 1;
+}
+#endif
