@@ -21,6 +21,8 @@
 ** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 ** See the GNU General Public License for more details.
 */
+#include "netstats.h"
+
 #define	MAXCPU		64
 #define	MAXDSK		128
 #define	MAXDKNAM	32
@@ -50,6 +52,16 @@ struct	memstat {
 /************************************************************************/
 
 struct	netstat {
+	struct ipv4_stats	ipv4;
+	struct icmpv4_stats	icmpv4;
+	struct udpv4_stats	udpv4;
+
+	struct ipv6_stats	ipv6;
+	struct icmpv6_stats	icmpv6;
+	struct udpv6_stats	udpv6;
+
+	struct tcp_stats	tcp;
+#if 0
 	count_t	tcpin;		/* TCP received packets		*/
 	count_t	tcpout;		/* TCP transmitted packets	*/
 
@@ -61,6 +73,7 @@ struct	netstat {
 	count_t	ipindel;	/* IP  local delivered packets	*/
 	count_t	ipfrw;		/* IP  forwarded packets	*/
 	count_t	cfuture[8];	/* reserved for future use	*/
+#endif
 };
 
 /************************************************************************/
@@ -79,9 +92,10 @@ struct percpu {
 };
 
 struct	cpustat {
+	count_t	nrcpu;	/* number of cpu's 			*/
 	count_t	devint;	/* number of device interrupts 		*/
 	count_t	csw;	/* number of context switches		*/
-	count_t	nrcpu;	/* number of cpu's 			*/
+	count_t	nprocs;	/* number of processes started          */
 	float	lavg1;	/* load average last    minute          */
 	float	lavg5;	/* load average last  5 minutes         */
 	float	lavg15;	/* load average last 15 minutes         */
@@ -96,9 +110,9 @@ struct	cpustat {
 struct	perxdsk {
         char	name[MAXDKNAM];	/* empty string for last        */
         count_t	nread;	/* number of read  transfers            */
-        count_t	nrblk;	/* number of sectors read               */
+        count_t	nrsect;	/* number of sectors read               */
         count_t	nwrite;	/* number of write transfers            */
-        count_t	nwblk;	/* number of sectors written            */
+        count_t	nwsect;	/* number of sectors written            */
         count_t	io_ms;	/* number of millisecs spent for I/O    */
         count_t	avque;	/* average queue length                 */
 	count_t	cfuture[4];	/* reserved for future use	*/
@@ -113,10 +127,27 @@ struct xdskstat {
 
 struct	perintf {
         char	name[16];	/* empty string for last        */
+
         count_t	rbyte;	/* number of read bytes                 */
         count_t	rpack;	/* number of read packets               */
+	count_t rerrs;  /* receive errors                       */
+	count_t rdrop;  /* receive drops                        */
+	count_t rfifo;  /* receive fifo                         */
+	count_t rframe; /* receive framing errors               */
+	count_t rcompr; /* receive compressed                   */
+	count_t rmultic;/* receive multicast                    */
+	count_t	rfuture[4];	/* reserved for future use	*/
+
         count_t	sbyte;	/* number of written bytes              */
         count_t	spack;	/* number of written packets            */
+	count_t serrs;  /* transmit errors                      */
+	count_t sdrop;  /* transmit drops                       */
+	count_t sfifo;  /* transmit fifo                        */
+	count_t scollis;/* collisions                           */
+	count_t scarrier;/* transmit carrier                    */
+	count_t scompr; /* transmit compressed                  */
+	count_t	sfuture[4];	/* reserved for future use	*/
+
 	long 	speed;	/* interface speed in megabits/second	*/
 	char	duplex;	/* full duplex (boolean) 		*/
 	count_t	cfuture[4];	/* reserved for future use	*/
@@ -127,8 +158,23 @@ struct intfstat {
 	struct perintf	intf[MAXINTF];
 };
 
-
 /************************************************************************/
+/*
+** experimental stuff for access to local HTTP daemons
+*/
+#define	HTTPREQ	"GET /server-status?auto HTTP/1.1\nHost: localhost\n\n"
+
+struct wwwstat {
+	count_t	accesses;	/* total number of HTTP-requests	*/
+	count_t	totkbytes;	/* total kbytes transfer for HTTP-req   */
+	count_t	uptime;		/* number of seconds since startup	*/
+	int	bworkers;	/* number of busy httpd-daemons		*/
+	int	iworkers;	/* number of idle httpd-daemons		*/
+};
+
+#if	HTTPSTATS
+int	getwwwstat(unsigned short, struct wwwstat *);
+#endif
 /************************************************************************/
 
 struct	sstat {
@@ -137,6 +183,8 @@ struct	sstat {
 	struct netstat	net;
 	struct intfstat	intf;
 	struct xdskstat xdsk;
+
+	struct wwwstat	www;
 };
 
 /*
