@@ -26,6 +26,9 @@
 ** --------------------------------------------------------------------------
 **
 ** $Log: rawlog.c,v $
+** Revision 1.21  2007/08/16 12:01:25  gerlof
+** Add support for atopsar reporting.
+**
 ** Revision 1.20  2007/03/20 13:02:25  gerlof
 ** Introduction of variable supportflags.
 **
@@ -94,7 +97,7 @@
 **
 */
 
-static const char rcsid[] = "$Id: rawlog.c,v 1.20 2007/03/20 13:02:25 gerlof Exp $";
+static const char rcsid[] = "$Id: rawlog.c,v 1.21 2007/08/16 12:01:25 gerlof Exp $";
 
 #include <sys/types.h>
 #include <sys/param.h>
@@ -116,7 +119,7 @@ static const char rcsid[] = "$Id: rawlog.c,v 1.20 2007/03/20 13:02:25 gerlof Exp
 #include "photoproc.h"
 #include "photosyst.h"
 
-#define	BASEPATH	"/var/log/atop/"  
+#define	BASEPATH	"/var/log/"  
 
 /*
 ** structure which describes the raw file contents
@@ -145,8 +148,7 @@ struct rawheader {
 	unsigned short	rawreclen;	/* length of struct rawrecord    */
 	unsigned short	hertz;		/* clock interrupts per second   */
 	unsigned short	sfuture[5];	/* future use                    */
-
-	char		node[32];	/* name of this system           */
+	struct utsname	utsname;	/* info about this system        */
 	char		cfuture[8];	/* future use                    */
 
 	unsigned int	pagesize;	/* size of memory page (bytes)   */
@@ -331,8 +333,8 @@ rawwopen()
 			if (rh.aversion & 0x8000)
 			{
 				fprintf(stderr,
-					"(created by atop version %d.%d - "
-					"current atop version %d.%d)\n",
+					"(created by version %d.%d - "
+					"current version %d.%d)\n",
 					(rh.aversion >> 8) & 0x7f,
 					 rh.aversion & 0xff,
 					 getnumvers() >> 8,
@@ -372,8 +374,7 @@ rawwopen()
 	rh.hertz	= hertz;
 	rh.pagesize	= pagesize;
 
-	strncpy(rh.node, utsname.nodename, sizeof rh.node);
-	rh.node[sizeof rh.node -1] = '\0';
+	memcpy(&rh.utsname, &utsname, sizeof rh.utsname);
 
 	if ( write(fd, &rh, sizeof rh) == -1)
 	{
@@ -420,7 +421,7 @@ rawread(unsigned int begintime, unsigned int endtime)
 		timenow	= time(0);
 		tp	= localtime(&timenow);
 
-		snprintf(rawname, RAWNAMESZ, "%s/atop_%04d%02d%02d",
+		snprintf(rawname, RAWNAMESZ, "%s/atop.log",
 			BASEPATH, 
 			tp->tm_year+1900,
 			tp->tm_mon+1,
@@ -499,8 +500,8 @@ rawread(unsigned int begintime, unsigned int endtime)
 
 	if (rh.magic != MYMAGIC)
 	{
-		fprintf(stderr, "file %s does not contain raw atop output "
-				"(wrong magic number)\n", rawname);
+		fprintf(stderr, "file %s does not contain raw atop/atopsar "
+				"output (wrong magic number)\n", rawname);
 		cleanstop(7);
 	}
 
@@ -518,8 +519,8 @@ rawread(unsigned int begintime, unsigned int endtime)
 		if (rh.aversion & 0x8000)
 		{
 			fprintf(stderr,
-				"(created by atop version %d.%d - "
-				"current atop version %d.%d)\n",
+				"(created by version %d.%d - "
+				"current version %d.%d)\n",
 				(rh.aversion >> 8) & 0x7f,
 				 rh.aversion & 0xff,
 				 getnumvers() >> 8,
@@ -529,9 +530,7 @@ rawread(unsigned int begintime, unsigned int endtime)
 		cleanstop(7);
 	}
 
-	strncpy(utsname.nodename, rh.node, sizeof utsname.nodename);
-	utsname.nodename[sizeof utsname.nodename -1] = '\0';
-
+	memcpy(&utsname, &rh.utsname, sizeof utsname);
 	supportflags = rh.supportflags;
 	osrel        = rh.osrel;
 	osvers       = rh.osvers;
@@ -549,7 +548,7 @@ rawread(unsigned int begintime, unsigned int endtime)
 	*/
 	if ( (offlist = malloc(sizeof(off_t) * OFFCHUNK)) == NULL)
 	{
-		perror("atop malloc");
+		perror("atop/atopsar malloc");
 		cleanstop(7);
 	}
 
@@ -561,6 +560,8 @@ rawread(unsigned int begintime, unsigned int endtime)
 	/*
 	** read a raw record header until end-of-file
 	*/
+	sampcnt = 0;
+
 	while (lastcmd && lastcmd != 'q')
 	{
 		while ( getrawrec(rawfd, &rr, rh.rawreclen) == rh.rawreclen)
@@ -579,7 +580,7 @@ rawread(unsigned int begintime, unsigned int endtime)
 				if ( (offlist = realloc(offlist,
 				     (offsize+OFFCHUNK)*sizeof(off_t))) ==NULL)
 				{
-					perror("atop realloc");
+					perror("atop/atopsar realloc");
 					cleanstop(7);
 				}
 
@@ -612,7 +613,7 @@ rawread(unsigned int begintime, unsigned int endtime)
 			if ( (devpstat =
 			       malloc(sizeof(struct pstat) * rr.nlist)) ==NULL)
 			{
-				perror("atop malloc");
+				perror("atop/atopsar malloc");
 				cleanstop(7);
 			}
 
@@ -624,6 +625,8 @@ rawread(unsigned int begintime, unsigned int endtime)
 			** activate the installed print-function to visualize
 			** the system- and process-level statistics
 			*/
+			sampcnt++;
+
 			lastcmd = (vis.show_samp)(rr.curtime, rr.interval,
 						&devsstat,    devpstat,
 					 	rr.nlist,     rr.npresent,
@@ -655,8 +658,9 @@ rawread(unsigned int begintime, unsigned int endtime)
 		lseek(rawfd, *(offlist+offcur), SEEK_SET);
 	}
 
+	free(offlist);
+
 	close(rawfd);
-	cleanstop(0);
 }
 
 /*
@@ -680,7 +684,7 @@ getrawsstat(int rawfd, struct sstat *sp, int complen)
 
 	if ( (compbuf = malloc(complen)) == NULL)
 	{
-		perror("atop malloc");
+		perror("atop/atopsar malloc");
 		cleanstop(7);
 	}
 
@@ -708,7 +712,7 @@ getrawpstat(int rawfd, struct pstat *pp, int complen, int nlist)
 
 	if ( (compbuf = malloc(complen)) == NULL)
 	{
-		perror("atop malloc");
+		perror("atop/atopsar malloc");
 		cleanstop(7);
 	}
 
@@ -759,26 +763,23 @@ testcompval(int rv, char *func)
 		break;
 
 	   case Z_MEM_ERROR:
-		fprintf(stderr,
-		        "atop - %s: failed due to lack of memory\n",
-			func);
+		fprintf(stderr, "atop/atopsar - "
+		        "%s: failed due to lack of memory\n", func);
 		cleanstop(7);
 
 	   case Z_BUF_ERROR:
-		fprintf(stderr, 
-		        "atop - %s: failed due to lack of room in out buffer\n",
-			func);
+		fprintf(stderr, "atop/atopsar - "
+			"%s: failed due to lack of room in buffer\n", func);
 		cleanstop(7);
 
    	   case Z_DATA_ERROR:
-		fprintf(stderr,
-		        "atop - %s: failed due to corrupted/incomplete data\n",
-			func);
+		fprintf(stderr, "atop/atopsar - "
+		        "%s: failed due to corrupted/incomplete data\n", func);
 		cleanstop(7);
 
 	   default:
-		fprintf(stderr,
-		        "atop - %s: unexpected error %d\n", func, rv);
+		fprintf(stderr, "atop/atopsar - "
+		        "%s: unexpected error %d\n", func, rv);
 		cleanstop(7);
 	}
 }
