@@ -30,6 +30,28 @@
 ** --------------------------------------------------------------------------
 ** 
 ** $Log: showgeneric.c,v $
+** Revision 1.48  2007/03/22 10:12:17  gerlof
+** Support for io counters (>= kernel 2.6.20).
+**
+** Revision 1.47  2007/03/21 14:22:34  gerlof
+** Handle io counters maintained from 2.6.20
+**
+** Revision 1.46  2007/03/20 11:13:15  gerlof
+** Cosmetic changes.
+**
+** Revision 1.45  2007/03/09 12:39:59  gerlof
+** Do not allow 'N' and 'D' when kernel-patch is not installed.
+**
+** Revision 1.44  2007/02/13 10:31:53  gerlof
+** Removal of external declarations.
+**
+** Revision 1.43  2007/01/18 10:41:45  gerlof
+** Add support for colors.
+** Add support for automatic determination of most critical resource.
+**
+** Revision 1.42  2006/11/13 13:48:36  gerlof
+** Implement load-average counters, context-switches and interrupts.
+**
 ** Revision 1.41  2006/04/03 05:42:35  gerlof
 ** *** empty log message ***
 **
@@ -159,7 +181,7 @@
 **
 */
 
-static const char rcsid[] = "$Id: showgeneric.c,v 1.41 2006/04/03 05:42:35 gerlof Exp $";
+static const char rcsid[] = "$Id: showgeneric.c,v 1.48 2007/03/22 10:12:17 gerlof Exp $";
 
 #include <sys/types.h>
 #include <sys/param.h>
@@ -185,35 +207,37 @@ static const char rcsid[] = "$Id: showgeneric.c,v 1.41 2006/04/03 05:42:35 gerlo
 #include "photosyst.h"
 #include "showgeneric.h"
 
-extern unsigned long	interval;
-extern int		deviatonly;
-extern char		rawreadflag;
-extern struct utsname	utsname;
-extern char		flaglist[];
-
 struct selection procsel = {"", {USERSTUB, }, "", 0, { 0, }};
 
 static void	showhelp(int);
-
-static int	screen;     	/* boolean: screen control-codes needed */
 static int	paused;     	/* boolean: currently in pause-mode     */
 static int	fixedhead;	/* boolean: fixate header-lines         */
+static int	usecolors=1;	/* boolean: colors for high occupation	*/
 static int	avgval;		/* boolean: average values i.s.o. total */
 
-static int	(*curcompar)(const void *, const void *) = compcpu;
-static char	*sortorder = "CPU", showtype = MPROCGEN;
+static char	showtype  = MPROCGEN;
+static char	showorder = MSORTCPU;
 
 static int	maxcpulines = 999;  /* maximum cpu       lines          */
 static int	maxdsklines = 999;  /* maximum disk      lines          */
 static int	maxintlines = 999;  /* maximum interface lines          */
 
-char    	*genhdr = "ATOP - %-18.18s "
+static char    	*genhdr = "ATOP - %-18.18s "
                           "%s  %s     %12ld seconds elapsed\n";
 
 static int	cumusers(struct pstat *, struct pstat *, int);
 static int	cumprocs(struct pstat *, struct pstat *, int);
 static long	getnumval(char *, long, int);
 
+
+static int	(*procsort[])(const void *, const void *) = {
+			[MSORTCPU]=compcpu, 
+			[MSORTMEM]=compmem, 
+			[MSORTDSK]=compdsk, 
+			[MSORTNET]=compnet, 
+};
+
+
 /*
 ** print the deviation-counters on process- and system-level
 */
@@ -228,7 +252,7 @@ generic_samp(time_t curtime, int nsecs,
 	int		firstproc = 0, plistsz, alistsz, killpid, killsig;
 	int		lastchar;
 	char		format1[16], format2[16];
-	char		*statmsg = NULL;
+	char		*statmsg = NULL, curorder, autoorder;
 	struct passwd 	*pwd;
 	struct syscap	syscap;
 
@@ -238,68 +262,64 @@ generic_samp(time_t curtime, int nsecs,
 	if (callnr == 0)	/* first call ? */
 	{
 		/*
-		** check if default sortorder and/or showtype are overruled
+		** check if default sort order and/or showtype are overruled
 		** by commando-line flags
 		*/
 		for (i=0; flaglist[i]; i++)
 		{
 			switch (flaglist[i])
 			{
+			   case MSORTAUTO:
+				showorder = MSORTAUTO;
+				break;
+
 			   case MSORTCPU:
-				curcompar = compcpu;
-				sortorder = "CPU";
+				showorder = MSORTCPU;
 				break;
 
 			   case MSORTMEM:
-				curcompar = compmem;
-				sortorder = "MEM";
+				showorder = MSORTMEM;
 				break;
 
 			   case MSORTDSK:
-				curcompar = compdsk;
-				sortorder = "DSK";
+				showorder = MSORTDSK;
 				break;
 
 			   case MSORTNET:
-				curcompar = compnet;
-				sortorder = "NET";
+				showorder = MSORTNET;
 				break;
 
 			   case MPROCGEN:
 				showtype  = MPROCGEN;
-				curcompar = compcpu;
-				sortorder = "CPU";
+				showorder = MSORTCPU;
 				break;
 
 			   case MPROCMEM:
 				showtype  = MPROCMEM;
-				curcompar = compmem;
-				sortorder = "MEM";
+				showorder = MSORTMEM;
 				break;
 
 			   case MPROCSCH:
 				showtype  = MPROCSCH;
-				curcompar = compcpu;
-				sortorder = "CPU";
+				showorder = MSORTCPU;
 				break;
 
 			   case MPROCDSK:
-				if ( !(kernpatch & PATCHSTAT) )
+				if ( !(supportflags & (PATCHSTAT|IOSTAT)) )
 				{
 					fprintf(stderr,
-						"No kernel-patch installed "
-						"(no disk-statistics)\n");
+						"No disk-activity figures "
+					        "available; request ignored\n");
 					sleep(3);
 					break;
 				}
 
 				showtype  = MPROCDSK;
-				curcompar = compdsk;
-				sortorder = "DSK";
+				showorder = MSORTDSK;
 				break;
 
 			   case MPROCNET:
-				if ( !(kernpatch & PATCHSTAT) )
+				if ( !(supportflags & PATCHSTAT) )
 				{
 					fprintf(stderr,
 						"No kernel-patch installed "
@@ -309,8 +329,7 @@ generic_samp(time_t curtime, int nsecs,
 				}
 
 				showtype  = MPROCNET;
-				curcompar = compnet;
-				sortorder = "NET";
+				showorder = MSORTNET;
 				break;
 
 			   case MPROCVAR:
@@ -343,6 +362,13 @@ generic_samp(time_t curtime, int nsecs,
 					fixedhead=1;
 				break;
 
+			   case MCOLORS:
+				if (usecolors)
+					usecolors=0;
+				else
+					usecolors=1;
+				break;
+
 			   case MSYSLIMIT:
 				maxcpulines = 0;
 				maxdsklines = 5;
@@ -350,7 +376,7 @@ generic_samp(time_t curtime, int nsecs,
 				break;
 
 			   case MVERSION:
-				printf("%s\n", getversion());
+				printf("%s\n", getstrvers());
 				cleanstop(0);
 
 			   default:
@@ -393,6 +419,20 @@ generic_samp(time_t curtime, int nsecs,
 				sleep(3);
 				cleanstop(1);
 			}
+
+			if (has_colors())
+			{
+				use_default_colors();
+				start_color();
+
+				init_pair(COLORLOW,  COLOR_GREEN,  -1);
+				init_pair(COLORMED,  COLOR_CYAN,   -1);
+				init_pair(COLORHIGH, COLOR_RED,    -1);
+			}
+			else
+			{
+				usecolors = 0;
+			}
 		} 
                 signal(SIGINT,   cleanstop);
                 signal(SIGTERM,  cleanstop);
@@ -464,7 +504,13 @@ generic_samp(time_t curtime, int nsecs,
 		/*
 		** print other lines of system-wide statistics
 		*/
-		curline = prisyst(sstat, curline, nsecs, avgval, fixedhead,
+		if (showorder == MSORTAUTO)
+			autoorder = MSORTCPU;
+		else
+			autoorder = showorder;
+
+		curline = prisyst(sstat, curline, nsecs, avgval,
+				fixedhead, usecolors, &autoorder,
 				maxcpulines, maxdsklines, maxintlines);
 
 		if (screen && curline+2 > LINES)
@@ -554,18 +600,27 @@ generic_samp(time_t curtime, int nsecs,
 		** sort the list in required order 
 		** (default CPU-consumption) and print the list
 		*/
+		if (showorder == MSORTAUTO)
+			curorder = autoorder;
+		else
+			curorder = showorder;
+
 		if (nact > 0 && plistsz > 0)
 		{
-			qsort(pstat, nact, sizeof(struct pstat), curcompar);
+			qsort(pstat, nact, sizeof(struct pstat),
+						procsort[(int)curorder]);
 
 			if (screen)
 				attron(A_REVERSE);
 
 			/*
 			** print the header
+			** first determine the column-header for the current
+			** sorting order of processes
 			*/
 			priphead(firstproc/plistsz+1, (nact-1)/plistsz+1,
-			        			sortorder, showtype);
+			       		showtype, curorder,
+					showorder == MSORTAUTO ? 1 : 0);
 
 			if (screen)
 			{
@@ -578,8 +633,8 @@ generic_samp(time_t curtime, int nsecs,
 			*/
 			priproc(pstat, firstproc, nact, curline+2,
 			        firstproc/plistsz+1, (nact-1)/plistsz+1,
-			        sortorder, &syscap, showtype, 
-				&procsel, screen, nsecs, avgval);
+			        showtype, curorder, &syscap, &procsel,
+				nsecs, avgval);
 		}
 
 		alistsz = nact;		/* preserve size of active list */
@@ -663,12 +718,20 @@ generic_samp(time_t curtime, int nsecs,
 
 				return lastchar;
 
+			   /*
+			   ** sort order automatically depending on
+			   ** most busy resource
+			   */
+			   case MSORTAUTO:
+				showorder = MSORTAUTO;
+				firstproc = 0;
+				break;
+
 			   /*
 			   ** sort in cpu-activity order
 			   */
 			   case MSORTCPU:
-				curcompar = compcpu;
-				sortorder = "CPU";
+				showorder = MSORTCPU;
 				firstproc = 0;
 				break;
 
@@ -676,8 +739,7 @@ generic_samp(time_t curtime, int nsecs,
 			   ** sort in memory-consumption order
 			   */
 			   case MSORTMEM:
-				curcompar = compmem;
-				sortorder = "MEM";
+				showorder = MSORTMEM;
 				firstproc = 0;
 				break;
 
@@ -685,8 +747,13 @@ generic_samp(time_t curtime, int nsecs,
 			   ** sort in disk-activity order
 			   */
 			   case MSORTDSK:
-				curcompar = compdsk;
-				sortorder = "DSK";
+				if ( !(supportflags & (PATCHSTAT|IOSTAT)) )
+				{
+					statmsg = "No disk-activity figures "
+					          "available; request ignored!";
+					break;
+				}
+				showorder = MSORTDSK;
 				firstproc = 0;
 				break;
 
@@ -694,8 +761,13 @@ generic_samp(time_t curtime, int nsecs,
 			   ** sort in network-activity order
 			   */
 			   case MSORTNET:
-				curcompar = compnet;
-				sortorder = "NET";
+				if ( !(supportflags & PATCHSTAT) )
+				{
+					statmsg = "No kernel-patch installed; "
+					          "request ignored!";
+					break;
+				}
+				showorder = MSORTNET;
 				firstproc = 0;
 				break;
 
@@ -704,8 +776,10 @@ generic_samp(time_t curtime, int nsecs,
 			   */
 			   case MPROCGEN:
 				showtype  = MPROCGEN;
-				curcompar = compcpu;
-				sortorder = "CPU";
+
+				if (showorder != MSORTAUTO)
+					showorder = MSORTCPU;
+
 				firstproc = 0;
 				break;
 
@@ -714,8 +788,10 @@ generic_samp(time_t curtime, int nsecs,
 			   */
 			   case MPROCMEM:
 				showtype  = MPROCMEM;
-				curcompar = compmem;
-				sortorder = "MEM";
+
+				if (showorder != MSORTAUTO)
+					showorder = MSORTMEM;
+
 				firstproc = 0;
 				break;
 
@@ -723,15 +799,17 @@ generic_samp(time_t curtime, int nsecs,
 			   ** disk-specific figures per process
 			   */
 			   case MPROCDSK:
-				if ( !(kernpatch & PATCHSTAT) )
+				if ( !(supportflags & (PATCHSTAT|IOSTAT)) )
 				{
-					statmsg = "No kernel-patch installed; "
-					          "request ignored!";
+					statmsg = "No disk-activity figures "
+					          "available; request ignored!";
 					break;
 				}
 				showtype  = MPROCDSK;
-				curcompar = compdsk;
-				sortorder = "DSK";
+
+				if (showorder != MSORTAUTO)
+					showorder = MSORTDSK;
+
 				firstproc = 0;
 				break;
 
@@ -739,15 +817,17 @@ generic_samp(time_t curtime, int nsecs,
 			   ** network-specific figures per process
 			   */
 			   case MPROCNET:
-				if ( !(kernpatch & PATCHSTAT) )
+				if ( !(supportflags & PATCHSTAT) )
 				{
 					statmsg = "No kernel-patch installed; "
 					          "request ignored!";
 					break;
 				}
 				showtype  = MPROCNET;
-				curcompar = compnet;
-				sortorder = "NET";
+
+				if (showorder != MSORTAUTO)
+					showorder = MSORTNET;
+
 				firstproc = 0;
 				break;
 
@@ -772,8 +852,10 @@ generic_samp(time_t curtime, int nsecs,
 			   */
 			   case MPROCSCH:
 				showtype  = MPROCSCH;
-				curcompar = compcpu;
-				sortorder = "CPU";
+
+				if (showorder != MSORTAUTO)
+					showorder = MSORTCPU;
+
 				firstproc = 0;
 				break;
 
@@ -944,9 +1026,23 @@ generic_samp(time_t curtime, int nsecs,
 
 					if (u == 0)
 					{
-						statmsg = "No user-names "
-						    "matching this pattern!";
-						beep();
+						/*
+						** possibly a numerical
+						** value has been specified
+						*/
+						if (numeric(procsel.username))
+						{
+						     procsel.userid[0] =
+							atoi(procsel.username);
+						     procsel.userid[1] =
+							USERSTUB;
+						}
+						else
+						{
+						     statmsg = "No user-names "
+						    	"match this pattern!";
+						     beep();
+						}
 					}
 				}
 				else
@@ -1076,6 +1172,33 @@ generic_samp(time_t curtime, int nsecs,
 				firstproc = 0;
 				break;
 
+			   /*
+			   ** screen lines:
+			   **	         toggle for colors
+			   */
+			   case MCOLORS:
+				if (usecolors)
+				{
+					usecolors=0;
+					statmsg = "No colors will be used...";
+				}
+				else
+				{
+					if (screen && has_colors())
+					{
+						usecolors=1;
+						statmsg =
+						   "Colors will be used...";
+					}
+					else
+					{
+						statmsg="No colors supported!";
+					}
+				}
+
+				firstproc = 0;
+				break;
+
 			   /*
 			   ** system-statistics lines:
 			   **	         toggle no or all active disk
@@ -1116,7 +1239,7 @@ generic_samp(time_t curtime, int nsecs,
 			   ** show version info
 			   */
 			   case MVERSION:
-				statmsg = getversion();
+				statmsg = getstrvers();
 				break;
 
 			   /*
@@ -1359,11 +1482,12 @@ static struct helptext {
 	{"\t'%c' - total resource consumption per program (i.e. same "
 	 "process name)\n",					MCUMPROC},
 	{"\n",							' '},
-	{"Sort list of active processes by:\n",			' '},
-	{"\t'%c' - cpu activity (default)\n",			MSORTCPU},
+	{"Sort list of active processes in order of:\n",	' '},
+	{"\t'%c' - cpu activity\n",				MSORTCPU},
 	{"\t'%c' - memory consumption\n",			MSORTMEM},
 	{"\t'%c' - disk activity\n",				MSORTDSK},
 	{"\t'%c' - network activity\n",				MSORTNET},
+	{"\t'%c' - most active system resource (auto mode)\n",	MSORTAUTO},
 	{"\n",							' '},
 	{"Screen-handling:\n",					' '},
 	{"\t^F  - show next     page in the process-list (forward)\n",	' '},
@@ -1389,6 +1513,8 @@ static struct helptext {
 								MPAUSE},
 	{"\t'%c' - fixate on static range of header-lines           (toggle)\n",
 								MSYSFIXED},
+	{"\t'%c' - use colors to indicate high occupation           (toggle)\n",
+								MCOLORS},
 	{"\t'%c' - show average-per-second i.s.o. total values      (toggle)\n",
 								MAVGVAL},
 	{"\t'%c' - limited lines for per-cpu, disk and interface resources\n",
@@ -1478,6 +1604,14 @@ generic_end(void)
 void
 generic_usage(void)
 {
+	printf("\t  -%c  show fixed number of lines with system-statistics\n",
+			MSYSFIXED);
+	printf("\t  -%c  show limited number of lines for certain resources\n",
+			MSYSLIMIT);
+	printf("\t  -%c  show average-per-second i.s.o. total values\n\n",
+			MAVGVAL);
+	printf("\t  -%c  no colors in case of high occupation\n",
+			MCOLORS);
 	printf("\t  -%c  show general process-info (default)\n",
 			MPROCGEN);
 	printf("\t  -%c  show memory-related process-info\n",
@@ -1492,24 +1626,23 @@ generic_usage(void)
 			MPROCVAR);
 	printf("\t  -%c  show command-line per process\n",
 			MPROCARG);
-	printf("\t  -%c  show totals per user\n",
+	printf("\t  -%c  show cumulated process-info per user\n",
 			MCUMUSER);
-	printf("\t  -%c  show totals per program (i.e. same process name)\n",
+	printf("\t  -%c  show cumulated process-info per program "
+	                "(i.e. same name)\n\n",
 			MCUMPROC);
-	printf("\t  -%c  sort by cpu-consumption (default)\n",
+	printf("\t  -%c  sort processes in order of cpu-consumption "
+	                "(default)\n",
 			MSORTCPU);
-	printf("\t  -%c  sort by memory-consumption\n",
+	printf("\t  -%c  sort processes in order of memory-consumption\n",
 			MSORTMEM);
-	printf("\t  -%c  sort by disk-activity\n",
+	printf("\t  -%c  sort processes in order of disk-activity\n",
 			MSORTDSK);
-	printf("\t  -%c  sort by network-activity\n",
+	printf("\t  -%c  sort processes in order of network-activity\n",
 			MSORTNET);
-	printf("\t  -%c  show fixed number of system-statistics\n",
-			MSYSFIXED);
-	printf("\t  -%c  limited lines for system-resources\n",
-			MSYSLIMIT);
-	printf("\t  -%c  show average-per-second i.s.o. total values\n",
-			MAVGVAL);
+	printf("\t  -%c  sort processes in order of most active resource "
+                        "(auto mode)\n",
+			MSORTAUTO);
 }
 
 /*
@@ -1547,10 +1680,21 @@ do_username(char *val)
 
 		if (u == 0)
 		{
-			fprintf(stderr,
-			        ".atoprc: user-names matching %s do not "
-				"exist\n", val);
-			exit(1);
+			/*
+			** possibly a numerical value has been specified
+			*/
+			if (numeric(procsel.username))
+			{
+			     procsel.userid[0] = atoi(procsel.username);
+			     procsel.userid[1] = USERSTUB;
+			}
+			else
+			{
+				fprintf(stderr,
+			       		".atoprc: user-names matching %s do "
+					"not exist\n", val);
+				exit(1);
+			}
 		}
 	}
 	else
@@ -1624,47 +1768,43 @@ do_flags(char *val)
 			break;
 
 		   case MSORTCPU:
-			curcompar = compcpu;
-			sortorder = "CPU";
+			showorder = MSORTCPU;
 			break;
 
 		   case MSORTMEM:
-			curcompar = compmem;
-			sortorder = "MEM";
+			showorder = MSORTMEM;
 			break;
 
 		   case MSORTDSK:
-			curcompar = compdsk;
-			sortorder = "DSK";
+			showorder = MSORTDSK;
 			break;
 
 		   case MSORTNET:
-			curcompar = compnet;
-			sortorder = "NET";
+			showorder = MSORTNET;
+			break;
+
+		   case MSORTAUTO:
+			showorder = MSORTAUTO;
 			break;
 
 		   case MPROCGEN:
 			showtype  = MPROCGEN;
-			curcompar = compcpu;
-			sortorder = "CPU";
+			showorder = MSORTCPU;
 			break;
 
 		   case MPROCMEM:
 			showtype  = MPROCMEM;
-			curcompar = compmem;
-			sortorder = "MEM";
+			showorder = MSORTMEM;
 			break;
 
 		   case MPROCDSK:
 			showtype  = MPROCDSK;
-			curcompar = compdsk;
-			sortorder = "DSK";
+			showorder = MSORTDSK;
 			break;
 
 		   case MPROCNET:
 			showtype  = MPROCNET;
-			curcompar = compnet;
-			sortorder = "NET";
+			showorder = MSORTNET;
 			break;
 
 		   case MPROCVAR:
@@ -1673,8 +1813,7 @@ do_flags(char *val)
 
 		   case MPROCSCH:
 			showtype  = MPROCSCH;
-			curcompar = compcpu;
-			sortorder = "CPU";
+			showorder = MSORTCPU;
 			break;
 
 		   case MPROCARG:
@@ -1700,6 +1839,10 @@ do_flags(char *val)
 		   case MSYSFIXED:
 			fixedhead=1;
 			break;
+
+		   case MCOLORS:
+			usecolors=0;
+			break;
 		}
 	}
 }
