@@ -29,6 +29,28 @@
 ** --------------------------------------------------------------------------
 **
 ** $Log: showlinux.c,v $
+** Revision 1.46  2007/11/07 09:23:29  gerlof
+** Modified format for avg1, avg5 and avg15 (CPL) when counters too large.
+**
+** Revision 1.45  2007/11/05 11:43:25  gerlof
+** Bug-solution for new-process indicator on 64-bits machines.
+**
+** Revision 1.44  2007/11/05 10:57:56  gerlof
+** Bug-solution for huge exit code on 64-bits machines.
+**
+** Revision 1.43  2007/08/17 09:45:57  gerlof
+** Experimental: gather info about HTTP statistics.
+**
+** Revision 1.42  2007/08/16 12:02:04  gerlof
+** Add support for atopsar reporting.
+** Concerns modification of networking-counters.
+**
+** Revision 1.41  2007/07/04 10:18:16  gerlof
+** Bug-solution for division by zero.
+**
+** Revision 1.40  2007/07/03 09:02:29  gerlof
+** Support Apache-statistics.
+**
 ** Revision 1.39  2007/03/22 10:12:54  gerlof
 ** Support for io counters (>= kernel 2.6.20).
 **
@@ -161,7 +183,7 @@
 **
 */
 
-static const char rcsid[] = "$Id: showlinux.c,v 1.39 2007/03/22 10:12:54 gerlof Exp $";
+static const char rcsid[] = "$Id: showlinux.c,v 1.46 2007/11/07 09:23:29 gerlof Exp $";
 
 #include <sys/types.h>
 #include <sys/param.h>
@@ -190,14 +212,14 @@ static const char rcsid[] = "$Id: showlinux.c,v 1.39 2007/03/22 10:12:54 gerlof
 ** critical percentages for occupation-percentage;
 ** these defaults can be overruled via the config-file
 */
-static int	cpubadness = 90;	/* percentage		*/
-static int	membadness = 90;	/* percentage		*/
-static int	swpbadness = 80;	/* percentage		*/
-static int	dskbadness = 70;	/* percentage		*/
-static int	netbadness = 90;	/* percentage		*/
-static int	pagbadness = 10;	/* number per second	*/
+int	cpubadness = 90;	/* percentage		*/
+int	membadness = 90;	/* percentage		*/
+int	swpbadness = 80;	/* percentage		*/
+int	dskbadness = 70;	/* percentage		*/
+int	netbadness = 90;	/* percentage		*/
+int	pagbadness = 10;	/* number per second	*/
 
-static int	almostcrit = 80;	/* percentage		*/
+int	almostcrit = 80;	/* percentage		*/
 
 /*
 ** table with column headers for sorted process list
@@ -230,14 +252,14 @@ char	*genprochdr2	= "\n  PID  SYSCPU  USRCPU  VGROW  RGROW  RDDSK  WRDSK "
 		     	  " ST EXC S %c%s CMD  %4d/%-4d\n";
 char 	*genproclina2	= "%5d %s %s %s %s %s %s  %c%c   - %c %3.0lf%% "
 	                  "%.14s\n";
-char 	*genprocline2	= "%s %s     0K     0K      -      -  %c%c %3ld "
+char 	*genprocline2	= "%s %s     0K     0K      -      -  %c%c %3d "
 			  "E %3.0lf%% <%.12s>\n";
 
 char	*genprochdr3	= "\n  PID  SYSCPU  USRCPU  VGROW  RGROW USERNAME THR  "
 		     	  "ST EXC  S %c%s CMD  %4d/%-4d\n";
 char 	*genproclina3	= "%5d %s %s %s %s %-8.8s %3d  %c%c   -  %c %3.0lf%% "
 	                  "%.14s\n";
-char 	*genprocline3	= "%s %s     0K     0K %-8.8s   0  %c%c %3ld  "
+char 	*genprocline3	= "%s %s     0K     0K %-8.8s   0  %c%c %3d  "
 			  "E %3.0lf%% <%.12s>\n";
 /***************************************************************************/
 char	*memprochdr	= "\n  PID MINFLT MAJFLT      VSTEXT "
@@ -275,7 +297,7 @@ char	*varprochdr	= "\n  PID  USERNAME GROUP       STDATE    STTIME      "
 	                  "ST EXC  S %c%s CMD  %4d/%-4d\n";
 char 	*varproclina	= "%5d  %-8.8s %-8.8s  %s %s     %c%c   -  %c %3.0lf%% "
 	                  "%.14s\n";
-char 	*varprocline	= " %-8.8s %-8.8s  %s %s     %c%c %3ld  E %3.0lf%% "
+char 	*varprocline	= " %-8.8s %-8.8s  %s %s     %c%c %3d  E %3.0lf%% "
 			  "<%.12s>\n";
 /***************************************************************************/
 char	*cmdprochdr	= "\n  PID %c%s COMMAND-LINE                           "
@@ -288,7 +310,7 @@ char	*schprochdr	= "\n  PID  TGID  THR  POLICY NICE PRIO RTPRIO "
 char 	*schproclina	= "%5d %5d %4d  %-6s %4d %4d %6d %6d %-2s   -   "
 			  "%c %3.0lf%% %.14s\n";
 char 	*schprocline	= "     -    0       -    -    -      -      - "
-			  "%c%c %3ld   E %3.0lf%% <%.12s>\n";
+			  "%c%c %3d   E %3.0lf%% <%.12s>\n";
 /***************************************************************************/
 char	*totuserhdr	= "\nNPROCS  SYSCPU  USRCPU  VSIZE  RSIZE  RDDSK "
 			  "WRDSK RNET SNET %c%s USER %4d/%-4d\n";
@@ -443,7 +465,7 @@ showgenproc(struct pstat *curstat, double perc, int nsecs, int avgval)
 						format5, KBFORMAT, 0, 0),
 				   val2memstr(curstat->dsk.wsz*512,
 						format6, KBFORMAT, 0, 0),
-			   	   curstat->gen.excode & ~(LONG_MAX) ?
+			   	   curstat->gen.excode & ~(INT_MAX) ?
 								'N' : '-',
 			   	   '-',
 			  	   curstat->gen.state,
@@ -479,7 +501,7 @@ showgenproc(struct pstat *curstat, double perc, int nsecs, int avgval)
 						format4, KBFORMAT, 0, 0),
 			   	username,
 			   	curstat->gen.nthr,
-			   	curstat->gen.excode & ~(LONG_MAX) ? 'N' : '-',
+			   	curstat->gen.excode & ~(INT_MAX) ? 'N' : '-',
 			   	'-',
 			   	curstat->gen.state,
 			   	perc, curstat->gen.name);
@@ -570,7 +592,7 @@ showgenproc(struct pstat *curstat, double perc, int nsecs, int avgval)
 								format1),
 			  	  val2cpustr(curstat->cpu.utime*1000/hertz,
 								format2),
-			  	  curstat->gen.excode & ~(LONG_MAX) ? 'N' : '-',
+			  	  curstat->gen.excode & ~(INT_MAX) ? 'N' : '-',
 			  	  exittype,
 		  	  	  curstat->gen.excode & 0xff        ? 
 		 		 	 curstat->gen.excode     & 0x7f :
@@ -601,7 +623,7 @@ showgenproc(struct pstat *curstat, double perc, int nsecs, int avgval)
 			  	  val2cpustr(curstat->cpu.utime*1000/hertz,
 								format2),
 			  	  username,
-			  	  curstat->gen.excode & ~(LONG_MAX) ? 'N' : '-',
+			  	  curstat->gen.excode & ~(INT_MAX) ? 'N' : '-',
 			  	  exittype,
 		  	  	  curstat->gen.excode & 0xff        ? 
 				 	 curstat->gen.excode     & 0x7f :
@@ -995,7 +1017,7 @@ showvarproc(struct pstat *curstat, double perc, int nsecs, int avgval)
 		  	groupname,
 			convdate(curstat->gen.btime, format1),
 			convtime(curstat->gen.btime, format2),
-			curstat->gen.excode & ~(LONG_MAX) ? 'N' : '-',
+			curstat->gen.excode & ~(INT_MAX) ? 'N' : '-',
 			'-',
 		  	curstat->gen.state,
 		  	perc, curstat->gen.name);
@@ -1027,7 +1049,7 @@ showvarproc(struct pstat *curstat, double perc, int nsecs, int avgval)
 		  	groupname,
 			convdate(curstat->gen.btime, format1),
 			convtime(curstat->gen.btime, format2),
-			curstat->gen.excode & ~(LONG_MAX) ? 'N' : '-',
+			curstat->gen.excode & ~(INT_MAX) ? 'N' : '-',
 			exittype,
 			curstat->gen.excode & 0xff        ?
 				curstat->gen.excode     & 0x7f :
@@ -1117,7 +1139,7 @@ showschproc(struct pstat *curstat, double perc, int nsecs, int avgval)
 		  	curstat->cpu.prio,
 		  	curstat->cpu.rtprio,
 		  	curstat->cpu.curcpu,
-			curstat->gen.excode & ~(LONG_MAX) ? "N-" : "--",
+			curstat->gen.excode & ~(INT_MAX) ? "N-" : "--",
 		  	curstat->gen.state,
 		  	perc, curstat->gen.name);
 	}
@@ -1144,7 +1166,7 @@ showschproc(struct pstat *curstat, double perc, int nsecs, int avgval)
 		}
 
 		printg(schprocline,
-			curstat->gen.excode & ~(LONG_MAX) ? 'N' : '-',
+			curstat->gen.excode & ~(INT_MAX) ? 'N' : '-',
 			exittype,
 		 	curstat->gen.excode & 0xff        ? 
 				 curstat->gen.excode     & 0x7f :
@@ -1441,6 +1463,7 @@ prisyst(struct sstat *sstat, int curline, int nsecs, int avgval,
 {
 	register int	i, lin;
 	count_t		cputot, percputot, mstot, busy;
+	count_t		udpin, udpout, ipin, ipout, ipfrw, ipindel;
 	unsigned int	badness, highbadness=0;
 	char		format1[16], format2[16], format3[16];
 	char		format4[16], format5[16];
@@ -1575,13 +1598,28 @@ prisyst(struct sstat *sstat, int curline, int nsecs, int avgval,
 	/*
 	** other CPU-related statistics
 	*/
-	printg("CPL | avg1 %6.2f | avg5 %7.2f | avg15 %6.2f | csw %s"
- 	       " | intr %s |\n",
-		sstat->cpu.lavg1,
-		sstat->cpu.lavg5,
-		sstat->cpu.lavg15,
-		val2valstr(sstat->cpu.csw,    format1, 8, avgval, nsecs),
-		val2valstr(sstat->cpu.devint, format2, 7, avgval, nsecs));
+	if (sstat->cpu.lavg1  > 999.0  ||
+	    sstat->cpu.lavg5  > 9999.0 ||
+	    sstat->cpu.lavg15 > 999.0    )
+	{
+		printg("CPL | avg1 %6.0f | avg5 %7.0f | avg15 %6.0f | csw %s"
+ 		       " | intr %s |\n",
+			sstat->cpu.lavg1,
+			sstat->cpu.lavg5,
+			sstat->cpu.lavg15,
+			val2valstr(sstat->cpu.csw,    format1, 8,avgval,nsecs),
+			val2valstr(sstat->cpu.devint, format2, 7,avgval,nsecs));
+	}
+	else
+	{
+		printg("CPL | avg1 %6.2f | avg5 %7.2f | avg15 %6.2f | csw %s"
+ 		       " | intr %s |\n",
+			sstat->cpu.lavg1,
+			sstat->cpu.lavg5,
+			sstat->cpu.lavg15,
+			val2valstr(sstat->cpu.csw,    format1, 8,avgval,nsecs),
+			val2valstr(sstat->cpu.devint, format2, 7,avgval,nsecs));
+	}
 
 	curline++;
 
@@ -1765,35 +1803,42 @@ prisyst(struct sstat *sstat, int curline, int nsecs, int avgval,
 	/*
 	** NET statistics
 	*/
-	if (sstat->net.tcpin || sstat->net.tcpout ||
-	    sstat->net.udpin || sstat->net.udpout || fixedhead )
+        udpin  = sstat->net.udpv4.InDatagrams  +
+			sstat->net.udpv6.Udp6InDatagrams;
+	udpout = sstat->net.udpv4.OutDatagrams +
+			sstat->net.udpv6.Udp6OutDatagrams;
+
+	if (sstat->net.tcp.InSegs || sstat->net.tcp.OutSegs ||
+	                   udpin ||                udpout || fixedhead )
 	{
 		printg("NET | transport   | tcpi %s | tcpo %s |"
 		       " udpi %s | udpo %s |\n",
-			val2valstr(sstat->net.tcpin,  format1, 7,
+			val2valstr(sstat->net.tcp.InSegs,  format1, 7,
 								avgval, nsecs),
-			val2valstr(sstat->net.tcpout, format2, 7,
+			val2valstr(sstat->net.tcp.OutSegs, format2, 7,
 								avgval, nsecs),
-			val2valstr(sstat->net.udpin,  format3, 7,
-								avgval, nsecs),
-			val2valstr(sstat->net.udpout, format4, 7,
-								avgval, nsecs));
-
+                        val2valstr(udpin,   format3, 7, avgval, nsecs),
+			val2valstr(udpout,  format4, 7, avgval, nsecs));
 		curline++;
 	}
 
-	if (sstat->net.ipin || sstat->net.ipout || fixedhead )
+	ipin    = sstat->net.ipv4.InReceives  +
+			sstat->net.ipv6.Ip6InReceives;
+	ipout   = sstat->net.ipv4.OutRequests +
+			sstat->net.ipv6.Ip6OutRequests;
+	ipindel = sstat->net.ipv4.InDelivers +
+			sstat->net.ipv6.Ip6InDelivers;
+	ipfrw   = sstat->net.ipv4.ForwDatagrams +
+			sstat->net.ipv6.Ip6OutForwDatagrams;
+
+	if (ipin || ipout || fixedhead )
 	{
 		printg("NET | network     | ipi %s | ipo %s | ipfrw %s |"
 		       " deliv %s |\n",
-			val2valstr(sstat->net.ipin,    format1, 8,
-								avgval, nsecs),
-			val2valstr(sstat->net.ipout,   format2, 8,
-								avgval, nsecs),
-			val2valstr(sstat->net.ipfrw,   format3, 6,
-								avgval, nsecs),
-			val2valstr(sstat->net.ipindel, format4, 6,
-								avgval, nsecs));
+			val2valstr(ipin,    format1, 8, avgval, nsecs),
+			val2valstr(ipout,   format2, 8, avgval, nsecs),
+			val2valstr(ipfrw,   format3, 6, avgval, nsecs),
+			val2valstr(ipindel, format4, 6, avgval, nsecs));
 
 		curline++;
 	}
@@ -1906,6 +1951,30 @@ prisyst(struct sstat *sstat, int curline, int nsecs, int avgval,
 		}
 	}
 
+	/*
+	** application statistics
+	**
+	** WWW: notice that we cause one access ourselves by fetching
+	**      the statistical counters
+	*/
+#if	HTTPSTATS
+	if (sstat->www.accesses > 1 || fixedhead )
+	{
+		printg("WWW | reqs %s | totKB %s | byt/rq %s | iwork %s |"
+		       " bwork %s |\n",
+			val2valstr(sstat->www.accesses,  format1, 6,
+							 avgval, nsecs),
+			val2valstr(sstat->www.totkbytes, format2, 6,
+						         avgval, nsecs),
+			val2valstr(sstat->www.accesses ?
+			    sstat->www.totkbytes*1024/sstat->www.accesses : 0,
+			                                 format3, 5, 0, 0),
+			val2valstr(sstat->www.iworkers,  format4, 6, 0, 0),
+			val2valstr(sstat->www.bworkers,  format5, 6, 0, 0) );
+		curline++;
+	}
+#endif
+
 	/*
 	** if the system is hardly loaded, still CPU-ordering of
 	** processes is most interesting (instead of memory)
